
#ifndef _RT2870_SOFTC_H_
#define _RT2870_SOFTC_H_

#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/sockio.h>
#include <sys/mbuf.h>
#include <sys/kernel.h>
#include <sys/socket.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/module.h>
#include <sys/bus.h>
#include <sys/endian.h>

#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>

#include <net/bpf.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <net/if_dl.h>
#include <net/if_media.h>
#include <net/if_types.h>

#include <net80211/ieee80211_var.h>
#include <net80211/ieee80211_amrr.h>
#include <net80211/ieee80211_radiotap.h>
#include <net80211/ieee80211_regdomain.h>

#include <dev/usb/usb.h>
#include <dev/usb/usbdi.h>
#include <dev/usb/usbdi_util.h>
#include "usbdevs.h"

#define RT2870_SOFTC_FLAGS_UCODE_LOADED						(1 << 0)

#define RT2870_SOFTC_AMRR_STA_COUNT							6

#define RT2870_SOFTC_LED_OFF_COUNT							3

#define RT2870_SOFTC_RSSI_OFF_COUNT							3

#define RT2870_SOFTC_LNA_GAIN_COUNT							4

#define RT2870_SOFTC_TXPOW_COUNT							50

#define RT2870_SOFTC_TXPOW_COMP_COUNT						5

#define RT2870_SOFTC_TSSI_COUNT								9

#define RT2870_SOFTC_BBP_EEPROM_COUNT						8

#define RT2870_SOFTC_TX_RING_COUNT							6

#define RT2870_SOFTC_RX_RING_DATA_COUNT						1
#define RT2870_SOFTC_TX_RING_DATA_COUNT						128

#define RT2870_SOFTC_CMD_DATA_LEN							256
#define RT2870_SOFTC_CMD_RING_CMD_COUNT						64

struct rt2870_softc_rx_data
{
	struct rt2870_softc *sc;
	usbd_xfer_handle xfer;
	uint8_t	*buf;
};

struct rt2870_softc_rx_ring
{
	int usb_ep;
	usbd_pipe_handle usb_pipe;
	struct rt2870_softc_rx_data data[RT2870_SOFTC_RX_RING_DATA_COUNT];
};

struct rt2870_softc_tx_data
{
	struct rt2870_softc *sc;
	struct ieee80211_node *ni;
	usbd_xfer_handle xfer;
	uint8_t	*buf;
	struct mbuf *m;
	int qid;
};

struct rt2870_softc_tx_ring
{
	int usb_ep;
	usbd_pipe_handle usb_pipe;
	struct rt2870_softc_tx_data data[RT2870_SOFTC_TX_RING_DATA_COUNT];
	int queued;
	int cur;
};

struct rt2870_softc_cmd
{
	void (*cb)(struct rt2870_softc *sc, void *arg);
	uint8_t	data[RT2870_SOFTC_CMD_DATA_LEN];
};

struct rt2870_softc_cmd_ring
{
	struct rt2870_softc_cmd cmd[RT2870_SOFTC_CMD_RING_CMD_COUNT];
	int	queued;
	int	cur;
	int	next;
};

struct rt2870_softc_rx_radiotap_header
{
	struct ieee80211_radiotap_header ihdr;
	uint8_t	flags;
	uint8_t	rate;
	uint16_t chan_freq;
	uint16_t chan_flags;
	uint8_t dbm_antsignal;
	uint8_t	antenna;
	uint8_t	antsignal;
};

#define RT2870_SOFTC_RX_RADIOTAP_PRESENT			\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_RATE) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL) |			\
	 (1 << IEEE80211_RADIOTAP_DBM_ANTSIGNAL) |		\
	 (1 << IEEE80211_RADIOTAP_ANTENNA) |			\
	 (1 << IEEE80211_RADIOTAP_DB_ANTSIGNAL))

struct rt2870_softc_tx_radiotap_header
{
	struct ieee80211_radiotap_header ihdr;
	uint8_t flags;
	uint8_t	rate;
	uint16_t chan_freq;
	uint16_t chan_flags;
};

#define RT2870_SOFTC_TX_RADIOTAP_PRESENT			\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_RATE) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL))

struct rt2870_softc
{
	uint32_t flags;

	device_t dev;

	usbd_device_handle usb_dev;
	usbd_interface_handle usb_iface;

	struct ifnet *ifp;

	struct ieee80211com ic;

	struct ieee80211_amrr amrr;
	struct ieee80211_amrr_node amrr_node;
	struct callout amrr_ch;
	uint32_t amrr_sta[RT2870_SOFTC_AMRR_STA_COUNT];
	usbd_xfer_handle amrr_xfer;

	int (*newstate)(struct ieee80211com *ic,
		enum ieee80211_state nstate, int arg);

	uint32_t mac_rev;
	uint16_t eeprom_rev;
	uint8_t rf_rev;

	uint8_t ntxpath;
	uint8_t nrxpath;

	int tx_agc_cntl;
	int ext_2ghz_lna;
	int ext_5ghz_lna;

	uint8_t country_2ghz;
	uint8_t country_5ghz;

	uint8_t rf_freq_off;

	uint8_t led_cntl;
	uint16_t led_off[RT2870_SOFTC_LED_OFF_COUNT];

	int8_t rssi_off_2ghz[RT2870_SOFTC_RSSI_OFF_COUNT];
	int8_t rssi_off_5ghz[RT2870_SOFTC_RSSI_OFF_COUNT];

	uint8_t lna_gain[RT2870_SOFTC_LNA_GAIN_COUNT];

	int tx_agc_cntl_2ghz;
	int tx_agc_cntl_5ghz;

	int8_t txpow1[RT2870_SOFTC_TXPOW_COUNT];
	int8_t txpow2[RT2870_SOFTC_TXPOW_COUNT];

	int8_t txpow_comp_delta_2ghz;
	int8_t txpow_comp_delta_5ghz;
	uint32_t txpow_comp_20mhz[RT2870_SOFTC_TXPOW_COMP_COUNT];
	uint32_t txpow_comp_40mhz_2ghz[RT2870_SOFTC_TXPOW_COMP_COUNT];
	uint32_t txpow_comp_40mhz_5ghz[RT2870_SOFTC_TXPOW_COMP_COUNT];

	uint8_t tssi_2ghz[RT2870_SOFTC_TSSI_COUNT];
	uint8_t tssi_step_2ghz;
	uint8_t tssi_5ghz[RT2870_SOFTC_TSSI_COUNT];
	uint8_t tssi_step_5ghz;

	struct
	{
		uint8_t	val;
		uint8_t	reg;
	} __packed bbp_eeprom[RT2870_SOFTC_BBP_EEPROM_COUNT];

	int sifs;

	struct rt2870_softc_rx_ring rx_ring;

	struct rt2870_softc_tx_ring tx_ring[RT2870_SOFTC_TX_RING_COUNT];
	int tx_ring_mgtqid;

	struct callout tx_watchdog_ch;
	int tx_timer;

	struct callout adjust_txpow_ch;

	struct ieee80211_beacon_offsets	beacon_offsets;

	struct rt2870_softc_cmd_ring cmd_ring;
	struct usb_task cmd_task;

	struct bpf_if *drvbpf;

	union
	{
		struct rt2870_softc_rx_radiotap_header th;
		uint8_t	pad[64];
	} rxtapu;

#define rxtap	rxtapu.th

	int	rxtap_len;

	union
	{
		struct rt2870_softc_tx_radiotap_header th;
		uint8_t	pad[64];
	} txtapu;

#define txtap	txtapu.th

	int	txtap_len;

#ifdef RT2870_DEBUG
	int debug;
#endif
};

#endif /* #ifndef _RT2870_SOFTC_H_ */
